package request

import (
	"context"
	"crypto/tls"
	"fmt"
	"log"

	"encoding/json"
	"io/ioutil"

	driver "github.com/arangodb/go-driver"
	"github.com/arangodb/go-driver/http"
)

// Service is a struct used to store this use case in
type Service struct {
}

// NewService creates a new instantion of this use case
func NewService() *Service {
	return &Service{}
}

// Document with Empty struct to retrieve all data from the DB Document
type Document map[string]interface{}

// GeneralFormat with Empty struct to retrieve all data from the DB Document
type GeneralFormat map[string][]Document

// ListContainer is a struct that keeps track of the nodes and edges that need to be returned
type ListContainer struct {
	nodeList []Document
	edgeList []Document
}

type arangoResult struct {
	vertices []Document
	edges    []Document
}

//attr interface{}

//map[1 , 2 , 3 map [ .. ]]

// SendAQLQuery send AQL string query to database and returns a JSON object in a general format
func (s *Service) SendAQLQuery(AQLQuery string) (*map[string][]Document, error) {
	var queryResult = make(map[string][]Document)
	conn, err := http.NewConnection(http.ConnectionConfig{
		Endpoints: []string{"https://aae8f5c054da.arangodb.cloud:8529"},
		TLSConfig: &tls.Config{InsecureSkipVerify: true},
	})
	if err != nil {
		log.Println("could not connect to database") // Handle error
		return nil, err
	}
	c, err := driver.NewClient(driver.ClientConfig{
		Connection:     conn,
		Authentication: driver.BasicAuthentication("root", "kXR0PFepIVm3EqgQ2MVc"),
	})
	if err != nil {
		log.Println("Could not log in to the ArangoDB") // Handle error
		return nil, err
	}

	ctx := context.Background()
	db, err := c.Database(ctx, "_system")
	if err != nil {
		// handle error
		log.Println(err)
		return nil, err
	}

	cursor, err := db.Query(ctx, AQLQuery, nil)
	if err != nil {
		log.Println("Invalid query") // handle error
		return nil, err
	}
	defer cursor.Close()

	listContainer := ListContainer{}
	for {
		var doc map[string][]interface{}
		_, err := cursor.ReadDocument(ctx, &doc)
		if driver.IsNoMoreDocuments(err) {
			break
		} else if err != nil {
			// handle other errors
			return nil, err
		}

		parseResult(doc, &listContainer)
	}

	queryResult["nodes"] = listContainer.nodeList
	queryResult["edges"] = listContainer.edgeList

	//writeJSON(queryResult)
	//file, err := json.MarshalIndent(queryResult, "", " ")
	return &queryResult, nil
}

//Resultaat: [ { vertices : [], edges : [] } ]
func parseResult(doc map[string][]interface{}, listContainer *ListContainer) {
	vertices := doc["vertices"]
	edges := doc["edges"]

	fmt.Println(vertices)

	for _, vertex := range vertices {
		vertexDoc := vertex.(map[string]interface{})

		(*listContainer).nodeList = append((*listContainer).nodeList, parseNode(vertexDoc))
	}

	for _, edge := range edges {
		edgeDoc := edge.(map[string]interface{})

		(*listContainer).edgeList = append((*listContainer).edgeList, parseEdge(edgeDoc))
	}
}

// parseResult takes the result of the query and translates this to two lists: a nodelist and an edgelist, stored in a listcontainer

// parseEdge parses the data of an edge to an output-friendly format
func parseEdge(d map[string]interface{}) Document {
	doc := d //.(map[string]interface{})

	data := make(Document)
	data["_id"] = doc["_id"]
	delete(doc, "_id")
	data["_key"] = doc["_key"]
	delete(doc, "_key")
	data["_rev"] = doc["_rev"]
	delete(doc, "_rev")
	data["_from"] = doc["_from"]
	delete(doc, "_from")
	data["_to"] = doc["_to"]
	delete(doc, "_to")
	data["attributes"] = doc

	//delete(doc, "_key")
	//data.rev = fmt.Sprintf("%v", doc["_rev"])

	// delete(doc, "_rev")
	// data.attr = doc
	return data
}

// writeJSON writes a json file for testing purposes
func writeJSON(queryResult map[string][]Document) {
	file, _ := json.MarshalIndent(queryResult, "", " ")

	_ = ioutil.WriteFile("result.json", file, 0644)
}

// parseNode parses the data of a node to an output-friendly format
func parseNode(d map[string]interface{}) Document {
	doc := d //.(map[string]interface{})

	data := make(Document)
	data["_id"] = doc["_id"]
	delete(doc, "_id")
	data["_key"] = doc["_key"]
	delete(doc, "_key")
	data["_rev"] = doc["_rev"]
	delete(doc, "_rev")
	data["attributes"] = doc

	//delete(doc, "_key")
	//data.rev = fmt.Sprintf("%v", doc["_rev"])

	// delete(doc, "_rev")
	// data.attr = doc
	return data
}
